@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2005-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2005-2013 uim Project https://github.com/uim/uim
 
   All rights reserved.
 
@@ -34,8 +34,10 @@
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#ifndef UIM_AGENT_CANDIDATE_H
-#define UIM_AGENT_CANDIDATE_H
+#ifndef UIM_EMACS_CANDIDATE_H
+#define UIM_EMACS_CANDIDATE_H
+
+#include <config.h>
 
 #include <stdlib.h>
 #include <string.h>
@@ -54,6 +56,9 @@ int new_candidate(uim_context context, candidate_info *cand,
 void clear_candidate(candidate_info *cand);
 
 int show_candidate(candidate_info *cand);
+#if UIM_EL_USE_NEW_PAGE_HANDLING
+void select_candidate(uim_context context, candidate_info *cand, int index);
+#endif
 
 void shift_candidate_page(uim_context context, candidate_info *cand, 
 						  int direction);
