@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2003-2013 uim Project https://github.com/uim/uim
 
   All rights reserved.
 
@@ -61,6 +61,9 @@
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
+#ifdef HAVE_TERMIOS_H
+#include <termios.h>
+#endif
 
 #include "uim-fep.h"
 #include "draw.h"
@@ -70,43 +73,42 @@
 
 #define my_putp(str) tputs(str, 1, my_putchar);
 
-
-/* TRUE */
+/* 初期化したらTRUE */
 static int s_init = FALSE;
-/* ߤΥ */
+/* 現在のカーソル位置 */
 static struct point_tag s_cursor = {UNDEFINED, UNDEFINED};
-/* ȼºݤΥκ */
+/* 取得したカーソルと実際のカーソルの差 */
 static struct point_tag s_cursor_diff = {0, 0};
-/* 뤬äƤȤTRUE */
+/* カーソルが消えているときTRUE */
 static int s_cursor_invisible = FALSE;
-/* ֤¸ȤTRUE */
+/* カーソル位置を保存したときTRUE */
 static int s_save = FALSE;
-/* ¸ */
+/* 保存したカーソル */
 static struct point_tag s_save_cursor;
-/* uim_modeˤʤȤ˽Ϥ륨ץ */
+/* uim_modeになるときに出力するエスケープシーケンス */
 static const char *s_enter_uim_mode;
-/* enter_underline_mode˴ޤޤ */
+/* enter_underline_modeに含まれる数字 */
 static const char *s_enter_underline_num;
-/* exit_underline_mode˴ޤޤ */
+/* exit_underline_modeに含まれる数字 */
 static const char *s_exit_underline_num;
-/* enter_standout_mode˴ޤޤ */
+/* enter_standout_modeに含まれる数字 */
 static const char *s_enter_standout_num;
-/* exit_standout_mode˴ޤޤ */
+/* exit_standout_modeに含まれる数字 */
 static const char *s_exit_standout_num;
-/* enter_bold_mode˴ޤޤ */
+/* enter_bold_modeに含まれる数字 */
 static const char *s_bold_num;
-/* enter_blink_mode˴ޤޤ */
+/* enter_blink_modeに含まれる数字 */
 static const char *s_blink_num;
-/* orig_pair˴ޤޤ */
+/* orig_pairに含まれる数字 */
 static const char *s_orig_pair_num;
-/* orig_pair˴ޤޤǽο */
+/* orig_pairに含まれる最初の数字 */
 static const char *s_orig_fore_num;
-/* orig_pair˴ޤޤ2ܤο */
+/* orig_pairに含まれる2番目の数字 */
 static const char *s_orig_back_num;
-/* ڤƤ륨ץ󥹤¸Хåե */
+/* 途中で切れているエスケープシーケンスを保存するバッファ */
 static char *s_escseq_buf = NULL;
 
-/* °ʤ */
+/* 属性なし */
 static const struct attribute_tag s_attr_none = {
   FALSE,     /* underline */
   FALSE,     /* standout */
@@ -116,9 +118,9 @@ static const struct attribute_tag s_attr_none = {
   FALSE      /* background */
 };
 
-/* uim⡼ɤ° */
+/* uimモードの属性 */
 static struct attribute_tag s_attr_uim;
-/* pty⡼ɤ° */
+/* ptyモードの属性 */
 static struct attribute_tag s_attr_pty = {
   FALSE,     /* underline */
   FALSE,     /* standout */
@@ -128,7 +130,7 @@ static struct attribute_tag s_attr_pty = {
   FALSE      /* background */
 };
 
-/* ߤ° */
+/* 現在の属性 */
 static struct attribute_tag s_attr = {
   FALSE,     /* underline */
   FALSE,     /* standout */
@@ -164,7 +166,7 @@ void init_escseq(const struct attribute_tag *attr_uim)
   escseq2n2(orig_pair, &s_orig_fore_num, &s_orig_back_num);
   s_enter_uim_mode = attr2escseq(&s_attr_uim);
   if (s_enter_uim_mode != NULL) {
-    s_enter_uim_mode = strdup(s_enter_uim_mode);
+    s_enter_uim_mode = uim_strdup(s_enter_uim_mode);
   }
 
   fixtty();
@@ -173,7 +175,7 @@ void init_escseq(const struct attribute_tag *attr_uim)
 }
 
 /*
- * termcap/terminfoΥȥ꤬뤫ǧ
+ * termcap/terminfoのエントリがあるか確認する
  */
 static void check_escseq(void)
 {
@@ -256,7 +258,7 @@ static void check_escseq(void)
 }
 
 /*
- * ^[[7m$<2>7Ф
+ * ^[[7m$<2>から7を取り出す
  */
 static char *escseq2n(const char *escseq)
 {
@@ -285,7 +287,7 @@ static char *escseq2n(const char *escseq)
 }
 
 /*
- * ^[[39;49m3949Ф
+ * ^[[39;49mから39と49を取り出す
  */
 static void escseq2n2(const char *escseq, const char **first, const char **second)
 {
@@ -343,7 +345,7 @@ void quit_escseq(void)
 }
 
 /*
- * üraw⡼ɤˤ
+ * 端末をrawモードにする
  */
 void fixtty(void)
 {
@@ -354,20 +356,20 @@ void fixtty(void)
 
   tcgetattr(g_win_in, &tios);
   cfmakeraw(&tios);
-  /* read_stdinޤǤɤޤʤФʤʤǾʸ */
+  /* read_stdinが戻るまでに読まなければならない最小の文字数 */
   tios.c_cc[VMIN] = 0;
-  /* read_stdinޤǤΥॢ 0.1ñ */
+  /* read_stdinが戻るまでのタイムアウト 0.1秒単位 */
   tios.c_cc[VTIME] = 3;
   tcsetattr(g_win_in, TCSANOW, &tios);
 
   if (s_init) {
     s_cursor.row = s_cursor.col = UNDEFINED;
-    /* λΰ֤ */
+    /* 前回終了時の位置に戻る */
     put_restore_cursor();
   }
 
   if (!g_opt.no_report_cursor) {
-    /* ϰ֤¸ */
+    /* 開始位置を保存 */
     start_cursor = get_cursor_position();
     if (start_cursor.row == UNDEFINED) {
       g_opt.no_report_cursor = TRUE;
@@ -389,25 +391,25 @@ void fixtty(void)
   }
 
   put_cursor_invisible();
-  /* ǲԤ鳫ϤȤΤ˥ */
+  /* 最下行から開始したときのためにスクロール */
   if (g_opt.status_type == LASTLINE) {
     write(g_win_out, "\n", strlen("\n"));
   }
 
   if (!s_init) {
     s_cursor.row = s_cursor.col = UNDEFINED;
-    /* ʰ֤˰ư */
+    /* 安全な位置に移動 */
     put_cursor_address(1, 1);
     s_cursor.row = s_cursor.col = UNDEFINED;
-    /* ֤ */
+    /* カーソル位置を取得 */
     cursor = get_cursor_position();
     s_cursor.row = s_cursor.col = UNDEFINED;
-    /* ֤˰ư */
+    /* 取得したカーソル位置に移動 */
     put_cursor_address_p(&cursor);
     s_cursor.row = s_cursor.col = UNDEFINED;
-    /* Ʊ֤뤫 */
+    /* 同じカーソル位置が得られるか */
     cursor2 = get_cursor_position();
-    /* ʤä麹ʬĴ٤ */
+    /* 得られなかったら差分を調べる */
     s_cursor_diff.row = cursor2.row - cursor.row;
     s_cursor_diff.col = cursor2.col - cursor.col;
     start_cursor.row -= s_cursor_diff.row;
@@ -418,7 +420,7 @@ void fixtty(void)
     put_change_scroll_region(0, g_win->ws_row - 1);
   }
   draw_statusline_force_no_restore();
-  /* ϰ֤ */
+  /* 開始位置に戻る */
   put_cursor_address_p(&start_cursor);
   put_cursor_normal();
 }
@@ -463,7 +465,7 @@ void put_cursor_right(int n)
 }
 
 /*
- * ֤¸
+ * カーソル位置を保存する
  */
 void put_save_cursor(void)
 {
@@ -481,7 +483,7 @@ void put_save_cursor(void)
 }
 
 /*
- * ¸֤
+ * 保存したカーソル位置を復元する
  */
 void put_restore_cursor(void)
 {
@@ -490,10 +492,10 @@ void put_restore_cursor(void)
     debug(("<put_restore_cursor>"));
     if (g_opt.no_report_cursor) {
       my_putp(restore_cursor);
-      /* DOSץץȤǤ1restore_cursorǤʤ */
+      /* DOSプロンプトでは1回のrestore_cursorでは戻らない */
       my_putp(restore_cursor);
       s_cursor = s_save_cursor;
-      /* DOSץץȤǤ°¸ʤ */
+      /* DOSプロンプトでは属性を保存しない */
       /* put_exit_attribute_mode(); */
     } else {
       put_cursor_address_p(&s_save_cursor);
@@ -509,9 +511,9 @@ do {                                     \
 } while(FALSE)
 
 /*
- * ߤΥ֤֤
- * 0, 0
- * ֤ʤäUNDEFINED֤
+ * 現在のカーソル位置を返す
+ * 左上は0, 0
+ * カーソル位置が得られなかったらUNDEFINEDを返す
  */
 struct point_tag get_cursor_position(void)
 {
@@ -548,7 +550,7 @@ struct point_tag get_cursor_position(void)
       escseq--;
     }
 
-    /* NULäƤ뤫⤷ʤΤ strchr ǤϤʤ memchr */
+    /* NULが入っているかもしれないので strchr ではなく memchr */
     while ((next_escseq = memchr(escseq + 1, ESCAPE_CODE, len - (escseq - ibuf) - 1)) != NULL) {
       int i = 1;
       int row = UNDEFINED;
@@ -638,17 +640,17 @@ struct point_tag get_cursor_position(void)
         continue;
       }
 
-      /* ץ󥹤ʸ󤬤뤫 */
+      /* エスケープシーケンスの前に文字列があるか */
       if (escseq > ibuf) {
         unget_stdin(ibuf, escseq - ibuf);
       }
 
-      /* ץ󥹤ʸ */
+      /* エスケープシーケンスの中の文字列 */
       if (unget_count > 0) {
         unget_stdin(unget_buf, unget_count);
       }
 
-      /* ץ󥹤θʸ */
+      /* エスケープシーケンスの後の文字列 */
       if (i < escseq_len - 1) {
         unget_stdin(&escseq[i + 1], escseq_len - 1 - i);
       }
@@ -661,7 +663,7 @@ struct point_tag get_cursor_position(void)
       s_cursor.row = row;
       s_cursor.col = col;
 
-      /* GNU screen ǤϤʤ뤳Ȥ */
+      /* GNU screen ではこうなることがある */
       if (s_cursor.col > g_win->ws_col - 1) {
         put_crlf();
       }
@@ -671,12 +673,12 @@ struct point_tag get_cursor_position(void)
   }
 
   unget_stdin(ibuf, len);
-  /*  */
+  /* 失敗 */
   return s_cursor;
 }
 
 /*
- * 򸫤ʤ褦ˤ
+ * カーソルを見えないようにする
  */
 void put_cursor_invisible(void)
 {
@@ -688,7 +690,7 @@ void put_cursor_invisible(void)
 }
 
 /*
- * 򸫤褦ˤ
+ * カーソルを見えるようにする
  */
 void put_cursor_normal(void)
 {
@@ -700,8 +702,8 @@ void put_cursor_normal(void)
 }
 
 /*
- * standout⡼ɡunderline⡼ɤλ
- * ʸطʿ򸵤᤹
+ * standoutモード、underlineモードを終了する
+ * 文字色、背景色を元に戻す
  */
 void put_exit_attribute_mode(void)
 {
@@ -711,7 +713,7 @@ void put_exit_attribute_mode(void)
 }
 
 /*
- * °fromtoѹfrom <= to
+ * 属性をfromからtoに変更し、from <= to
  */
 static void change_attr(struct attribute_tag *from, const struct attribute_tag *to)
 {
@@ -756,8 +758,8 @@ static void change_attr(struct attribute_tag *from, const struct attribute_tag *
 }
 
 /*
- * ط°fromtoѹfrom <= to
- * ʸѤʤ⤷ʤ
+ * 背景属性をfromからtoに変更し、from <= to
+ * 文字色は変わらないかもしれない
  */
 static void change_background_attr(struct attribute_tag *from, struct attribute_tag to)
 {
@@ -773,12 +775,13 @@ static void change_background_attr(struct attribute_tag *from, struct attribute_
 }
 
 /*
- * attrб륨ץ󥹤֤
- * ֤ͤŪʥХåե
+ * attrに対応するエスケープシーケンスを返す
+ * 返り値は静的なバッファ
  */
 static const char *attr2escseq(const struct attribute_tag *attr)
 {
-  static char escseq[20];
+  static char escseq[40];
+  char numstr[20];
   int add_semicolon = FALSE;
   if (!attr->underline && !attr->standout && !attr->bold && !attr->blink
       && attr->foreground == FALSE && attr->background == FALSE) {
@@ -787,73 +790,87 @@ static const char *attr2escseq(const struct attribute_tag *attr)
   strlcpy(escseq, "\033[", sizeof(escseq));
   if (attr->underline && s_enter_underline_num != NULL) {
     add_semicolon = TRUE;
-    strcat(escseq, s_enter_underline_num);
+    strlcat(escseq, s_enter_underline_num, sizeof(escseq));
   }
   if (attr->standout && s_enter_standout_num != NULL) {
     if (add_semicolon) {
-      strcat(escseq, ";");
+      strlcat(escseq, ";", sizeof(escseq));
     } else {
       add_semicolon = TRUE;
     }
-    strcat(escseq, s_enter_standout_num);
+    strlcat(escseq, s_enter_standout_num, sizeof(escseq));
   }
   if (attr->bold && s_bold_num != NULL) {
     if (add_semicolon) {
-      strcat(escseq, ";");
+      strlcat(escseq, ";", sizeof(escseq));
     } else {
       add_semicolon = TRUE;
     }
-    strcat(escseq, s_bold_num);
+    strlcat(escseq, s_bold_num, sizeof(escseq));
   }
   if (attr->blink && s_blink_num != NULL) {
     if (add_semicolon) {
-      strcat(escseq, ";");
+      strlcat(escseq, ";", sizeof(escseq));
     } else {
       add_semicolon = TRUE;
     }
-    strcat(escseq, s_blink_num);
+    strlcat(escseq, s_blink_num, sizeof(escseq));
   }
   if (attr->foreground != FALSE) {
     if (add_semicolon) {
-      strcat(escseq, ";");
+      strlcat(escseq, ";", sizeof(escseq));
     } else {
       add_semicolon = TRUE;
     }
-    sprintf(escseq + strlen(escseq), "%d", attr->foreground);
+    if (attr->foreground & EXT_COLOR_256) {
+      snprintf(numstr, sizeof(numstr), "38;5;%d", attr->foreground & 255);
+    } else if (attr->foreground & EXT_COLOR_24BIT) {
+      snprintf(numstr, sizeof(numstr), "38;2;%d;%d;%d", attr->foreground & 255, (attr->foreground >> 8) & 255, (attr->foreground >> 16) & 255);
+    } else {
+      snprintf(numstr, sizeof(numstr), "%d", attr->foreground);
+    }
+    strlcat(escseq, numstr, sizeof(escseq));
   }
   if (attr->background != FALSE) {
     if (add_semicolon) {
-      strcat(escseq, ";");
+      strlcat(escseq, ";", sizeof(escseq));
     }
-    sprintf(escseq + strlen(escseq), "%d", attr->background);
+    if (attr->background & EXT_COLOR_256) {
+      snprintf(numstr, sizeof(numstr), "48;5;%d", attr->background & 255);
+    } else if (attr->background & EXT_COLOR_24BIT) {
+      snprintf(numstr, sizeof(numstr), "48;2;%d;%d;%d", attr->background & 255, (attr->background >> 8) & 255, (attr->background >> 16) & 255);
+    } else {
+      snprintf(numstr, sizeof(numstr), "%d", attr->background);
+    }
+    strlcat(escseq, numstr, sizeof(escseq));
   }
-  strcat(escseq, "m");
-  debug2(("attr2escseq underline = %d standout = %d bold = %d blink = %d fore = %d back = %d\n",
+  strlcat(escseq, "m", sizeof(escseq));
+  debug2(("attr2escseq underline = %d standout = %d bold = %d blink = %d fore = %d(0x%x) back = %d(0x%x)\n",
       attr->underline, attr->standout, attr->bold, attr->blink,
-      attr->foreground, attr->background));
+      attr->foreground, attr->foreground, attr->background, attr->background));
   debug2(("attr2escseq = %s\n", escseq));
   return escseq;
 }
 
 /*
- * str°ѹ륨ץ󥹤õ
+ * strから属性を変更するエスケープシーケンスを探す
  * len > 0
  */
 static void set_attr(const char *str, int len)
 {
-  /* ץ󥹤γ */
+  /* エスケープシーケンスの開始 */
   const char *start;
   const char *free_str = NULL;
-  /* TRUEΤȤfree_strfreeɬפ */
+  /* TRUEのときfree_strをfreeする必要がある */
   int must_free = FALSE;
   const char *end = str + len;
-  /* ΥХåե뤫 */
+  /* 前回の途中のバッファがあるか */
   if (s_escseq_buf != NULL) {
     const char *tmp_str = str;
-    /* s_escseq_buf ˤ'\0'ϴޤޤʤ */
+    /* s_escseq_buf には'\0'は含まれない */
     int escseq_buf_len = strlen(s_escseq_buf);
     must_free = TRUE;
-    free_str = str = malloc(escseq_buf_len + len + 1);
+    free_str = str = uim_malloc(escseq_buf_len + len + 1);
     memcpy((char *)str, s_escseq_buf, escseq_buf_len);
     memcpy((char *)str + escseq_buf_len, tmp_str, len);
     ((char *)str)[escseq_buf_len + len] = '\0';
@@ -866,15 +883,15 @@ static void set_attr(const char *str, int len)
     str++;
     if (str == end) {
       int escseq_buf_len = end - start;
-      s_escseq_buf = malloc(escseq_buf_len + 1);
+      s_escseq_buf = uim_malloc(escseq_buf_len + 1);
       memcpy(s_escseq_buf, start, escseq_buf_len);
       s_escseq_buf[escseq_buf_len] = '\0';
     }
 
     else if (str[0] == '[') {
       int nr_params = 1;
-      /* ^[[1;2;31, 2, 3 */
-      int *params = malloc(sizeof(int));
+      /* ^[[1;2;3の1, 2, 3を入れる */
+      int *params = uim_malloc(sizeof(int));
       params[0] = 0;
       str++;
 
@@ -889,7 +906,7 @@ static void set_attr(const char *str, int len)
         }
         if (str[0] == ';') {
           nr_params++;
-          params = realloc(params, sizeof(int) * nr_params);
+          params = uim_realloc(params, sizeof(int) * nr_params);
           params[nr_params - 1] = 0;
           str++;
         }
@@ -897,7 +914,7 @@ static void set_attr(const char *str, int len)
 
       if (str == end) {
         int escseq_buf_len = end - start;
-        s_escseq_buf = malloc(escseq_buf_len + 1);
+        s_escseq_buf = uim_malloc(escseq_buf_len + 1);
         memcpy(s_escseq_buf, start, escseq_buf_len);
         s_escseq_buf[escseq_buf_len] = '\0';
       }
@@ -907,7 +924,7 @@ static void set_attr(const char *str, int len)
         str++;
         for (i = 0; i < nr_params; i++) {
           if (params[i] == 0) {
-            /* °õ */
+            /* 属性消去 */
             s_attr = s_attr_none;
           } else if (s_enter_underline_num != NULL && params[i] == atoi(s_enter_underline_num)) {
             s_attr.underline = TRUE;
@@ -935,6 +952,29 @@ static void set_attr(const char *str, int len)
             s_attr.foreground = params[i];
           } else if ((40 <= params[i] && params[i] <= 47) || (100 <= params[i] && params[i] <= 107)) {
             s_attr.background = params[i];
+          } else if (params[i] == 38 || params[i] == 48) {
+            int j;
+            int *ext_color_dst = params[i] == 38 ? &s_attr.foreground : &s_attr.background;
+            i++;
+            if (i < nr_params) {
+              if (params[i] == 2 || params[i] == 5) {
+                int nr_ext_params;
+                if (params[i] == 5) {
+                  *ext_color_dst = EXT_COLOR_256;
+                  nr_ext_params = 1;
+                } else {
+                  *ext_color_dst = EXT_COLOR_24BIT;
+                  nr_ext_params = 3;
+                }
+
+                for (j = 0; j < nr_ext_params && i < nr_params; j++) {
+                  i++;
+                  if (0 <= params[i] && params[i] <= 255) {
+                    *ext_color_dst |= params[i] << (j * 8);
+                  }
+                }
+              }
+            }
           }
         }
       }
@@ -948,8 +988,8 @@ static void set_attr(const char *str, int len)
 }
 
 /*
- * row col˰ư
- * 0, 0
+ * row行 col列に移動する
+ * 左上は0, 0
  */
 void put_cursor_address(int row, int col)
 {
@@ -957,11 +997,11 @@ void put_cursor_address(int row, int col)
   if (row >= g_win->ws_row) {
     row = g_win->ws_row - 1;
   }
-  /* üؤΰưϾάʤ */
+  /* 右端への移動は省略しない */
   if (row == s_cursor.row && col == s_cursor.col && col < g_win->ws_col - 2) {
     return;
   }
-  tmp = tparm(cursor_address, row, col);
+  tmp = tparm(cursor_address, row, col, 0, 0, 0, 0, 0, 0, 0);
   my_putp(tmp);
   s_cursor.row = row;
   s_cursor.col = col;
@@ -974,7 +1014,7 @@ void put_cursor_address_p(struct point_tag *p)
 }
 
 /*
- * nʸ
+ * n文字挿入する
  */
 void put_insert(int n)
 {
@@ -982,13 +1022,13 @@ void put_insert(int n)
   if (n <= 0) {
     return;
   }
-  tmp = tparm(parm_ich, n);
+  tmp = tparm(parm_ich, n, 0, 0, 0, 0, 0, 0, 0, 0);
   my_putp(tmp);
   debug(("<ins %d>", n));
 }
 
 /*
- * nʸõ
+ * n文字消去する
  */
 void put_delete(int n)
 {
@@ -1004,13 +1044,13 @@ void put_delete(int n)
     change_background_attr(&s_attr, s_attr_uim);
   }
 
-  tmp = tparm(parm_dch, n);
+  tmp = tparm(parm_dch, n, 0, 0, 0, 0, 0, 0, 0, 0);
   my_putp(tmp);
   debug(("<del %d>", n));
 }
 
 /*
- * Ԥ
+ * 改行する
  */
 void put_crlf(void)
 {
@@ -1024,7 +1064,7 @@ void put_crlf(void)
 }
 
 /*
- * ǲԤcol˰ư
+ * 最下行のcolに移動する
  */
 void put_goto_lastline(int col)
 {
@@ -1033,7 +1073,7 @@ void put_goto_lastline(int col)
   if (row == s_cursor.row && col == s_cursor.col) {
     return;
   }
-  tmp = tparm(cursor_address, row, col);
+  tmp = tparm(cursor_address, row, col, 0, 0, 0, 0, 0, 0, 0);
   my_putp(tmp);
   s_cursor.row = row;
   s_cursor.col = col;
@@ -1041,8 +1081,8 @@ void put_goto_lastline(int col)
 }
 
 /*
- * underline⡼ɤstandout⡼ɤλnʸڡϤ
- * ̤αüۤƤϤʤ
+ * underlineモードとstandoutモードを終了してn文字スペースを出力する
+ * 画面の右端を越えてはいけない
  */
 void put_erase(int n)
 {
@@ -1053,7 +1093,7 @@ void put_erase(int n)
     return;
   }
 
-  spaces = malloc(n + 1);
+  spaces = uim_malloc(n + 1);
   for (i = 0; i < n; i++) {
     spaces[i] = ' ';
   }
@@ -1077,8 +1117,8 @@ void put_erase(int n)
 }
 
 /*
- * underline⡼ɤstandout⡼ɤλƹޤǾõ
- * ֤Ѥʤ
+ * underlineモードとstandoutモードを終了して行末まで消去
+ * カーソル位置は変わらない
  */
 void put_clear_to_end_of_line(int width)
 {
@@ -1096,21 +1136,21 @@ void put_clear_to_end_of_line(int width)
 }
 
 /*
- * üΰstartendꤹ
+ * 端末の描画領域をstartからendに設定する
  */
 void put_change_scroll_region(int start, int end)
 {
-  const char *tmp = tparm(change_scroll_region, start, end);
+  const char *tmp = tparm(change_scroll_region, start, end, 0, 0, 0, 0, 0, 0, 0);
   my_putp(tmp);
   s_cursor.row = s_cursor.col = 0;
   debug(("<region %d %d>", start, end));
 }
 
 /*
- * strü˽Ϥ
- * ̤αüۤƤϤʤ
- * ץ󥹤ϴޤޤʤ
- * ȿžattrǻ
+ * strを端末に出力する
+ * 画面の右端を越えてはいけない
+ * エスケープシーケンスは含まない
+ * 下線、反転はattrで指定
  */
 void put_uim_str(const char *str, int attr)
 {
@@ -1133,9 +1173,9 @@ void put_uim_str(const char *str, int attr)
 }
 
 /*
- * strlenϤput_uim_str
- * const char *strȤʤäƤ뤬Ū˽񴹤Τstrʸ
- * ˤƤϤʤ
+ * strのlenだけ出力するput_uim_str
+ * const char *strとなっているが，一時的に書換えるのでstrを文字列定数
+ * にしてはいけない．
  */
 void put_uim_str_len(const char *str, int attr, int len)
 {
@@ -1146,7 +1186,7 @@ void put_uim_str_len(const char *str, int attr, int len)
 }
 
 /*
- * str򿧤ʤǽϤput_uim_str
+ * strを色なしで出力するput_uim_str
  */
 void put_uim_str_no_color(const char *str, int attr)
 {
@@ -1157,7 +1197,7 @@ void put_uim_str_no_color(const char *str, int attr)
 }
 
 /*
- * strlenϤput_uim_str_no_color
+ * strのlenだけ出力するput_uim_str_no_color
  */
 void put_uim_str_no_color_len(const char *str, int attr, int len)
 {
@@ -1168,8 +1208,8 @@ void put_uim_str_no_color_len(const char *str, int attr, int len)
 }
 
 /*
- * ptyνstrü˽Ϥ
- * ץ󥹤ޤ礬
+ * ptyからの出力strを端末に出力する
+ * エスケープシーケンスを含む場合がある
  */
 void put_pty_str(const char *str, int len)
 {
@@ -1187,8 +1227,8 @@ void put_pty_str(const char *str, int len)
 
 
 /*
- * escseq/$<\d+>/Τ֤
- * escseqNULLäNULL֤
+ * escseqから/$<\d+>/を取り除いたものを返す
+ * escseqがNULLだったらNULLを返す
  */
 char *cut_padding(const char *escseq)
 {
@@ -1197,7 +1237,7 @@ char *cut_padding(const char *escseq)
   if (escseq == NULL) {
     return NULL;
   }
-  cut_str = malloc(strlen(escseq) + 1);
+  cut_str = uim_malloc(strlen(escseq) + 1);
   for (i = 0, j = 0; escseq[i] != '\0';) {
     if (escseq[i] == '$' && escseq[i + 1] == '<') {
       int i2 = i + 1;
@@ -1216,7 +1256,7 @@ char *cut_padding(const char *escseq)
 }
 
 /*
- * üѹ줿Ȥ˸Ƥ
+ * 端末サイズが変更されたときに呼ぶ
  */
 void escseq_winch(void)
 {
