@@ -1,4 +1,4 @@
-#line 1 "../../../uim/sigscheme/test-c/test-global.c"
+#line 1 "/home/uim/source/sigscheme/test-c/test-global.c"
 /*===========================================================================
  *  Filename : test-global.c
  *  About    : unit test for global object handlings
