@@ -1,4 +1,4 @@
-#line 1 "../../../uim/sigscheme/test-c/test-length.c"
+#line 1 "/home/uim/source/sigscheme/test-c/test-length.c"
 /*===========================================================================
  *  Filename : test-length.c
  *  About    : test for list length -related functions
