@@ -1,4 +1,4 @@
-#line 1 "../../../uim/sigscheme/test-c/test-storage.c"
+#line 1 "/home/uim/source/sigscheme/test-c/test-storage.c"
 /*===========================================================================
  *  Filename : test-storage.c
  *  About    : scheme object representation and accessor test
