@@ -1,6 +1,6 @@
 /*
 
-  Copyright (c) 2003-2009 uim Project http://code.google.com/p/uim/
+  Copyright (c) 2003-2013 uim Project https://github.com/uim/uim
 
   All rights reserved.
 
@@ -36,12 +36,13 @@
 
 #define UIM_XIM_USE_JAPANESE_KANA_KEYBOARD_HACK 1
 
-#include <stdio.h>
-#include <ctype.h>
-#include <locale.h>
+#include <cctype>
+#include <clocale>
+#include <cstdio>
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
 #include <X11/Xutil.h>
+#include <X11/XKBlib.h>
 
 /* workaround for pre X11R6.7 */
 #ifndef XK_KOREAN
@@ -62,9 +63,7 @@
 #include "uim/uim-helper.h"
 #include "uim/uim-im-switcher.h"
 #include "uim/uim-scm.h"
-#if UIM_XIM_USE_JAPANESE_KANA_KEYBOARD_HACK
 #include "uim/uim-x-util.h"
-#endif
 
 #ifndef XK_dead_horn
 #define XK_dead_horn	0xfe62
@@ -84,6 +83,9 @@ static int check_modifier(std::list<KeySym> list);
 static int gMod1Mask, gMod2Mask, gMod3Mask, gMod4Mask, gMod5Mask;
 static int gXNumLockMask;
 
+CandWinPosType XimServer::gCandWinPosType;
+CandWinStyle XimServer::gCandWinStyle;
+bool XimServer::gCandWinStyleUpdated;
 
 void print_ustring(uString *s)
 {
@@ -175,6 +177,9 @@ void XimServer::customContext(const char *custom, const char *val) {
 
     if (!strcmp(custom, "candidate-window-position"))
 	check_candwin_pos_type();
+
+    if (!strcmp(custom, "candidate-window-style"))
+	check_candwin_style();
 }
 
 void XimServer::reloadConfigs() {
@@ -200,6 +205,7 @@ void XimServer::reloadConfigs() {
 	disp->hide_caret_state();
     }
 
+    check_candwin_style();
     check_candwin_pos_type();
 }
 
@@ -272,7 +278,7 @@ XimServer::strToUstring(uString *d, const char *s)
     int l = 0, nbyte = 0;
     uchar ch;
 
-    len = strlen(s);
+    len = static_cast<int>(strlen(s));
     while (l < len && *s != 0 &&
 	   (nbyte = utf8_mbtowc(&ch, (const unsigned char *)s, len - l)) > 0) {
 	    d->push_back(ch);
@@ -303,8 +309,7 @@ const char *XimServer::getIMLang()
 
 void XimServer::set_im(const char *engine)
 {
-    if (mIMName)
-	free(mIMName);
+    free(mIMName);
 
     mIMName = strdup(engine);
     mIMLang = get_im_lang_from_engine(engine);
@@ -343,6 +348,9 @@ InputContext::InputContext(XimServer *svr, XimIC *ic, const char *engine)
 
 InputContext::~InputContext()
 {
+#if UIM_XIM_USE_DELAY
+    timer_cancel();
+#endif
     if (mFocusedContext == this)
 	mFocusedContext = NULL;
 
@@ -432,6 +440,10 @@ InputContext::createUimContext(const char *engine)
 	uim_set_im_switch_request_cb(uc,
 			InputContext::switch_app_global_im_cb,
 			InputContext::switch_system_global_im_cb);
+#if UIM_XIM_USE_DELAY
+	uim_set_delay_candidate_selector_cb(uc,
+			InputContext::candidate_activate_with_delay_cb);
+#endif
 
 	if (mFocusedContext == this)
 	    uim_prop_list_update(uc);
@@ -492,7 +504,10 @@ void InputContext::switch_system_global_im(const char *name)
     for (it = XimServer::gServerMap.begin(); it != XimServer::gServerMap.end(); ++it)
 	(*it).second->changeContext(name);
 
-    asprintf(&msg, "im_change_whole_desktop\n%s\n", name);
+    if (asprintf(&msg, "im_change_whole_desktop\n%s\n", name) == -1) {
+        free(msg);
+        return;
+    }
     uim_helper_send_message(lib_uim_fd, msg);
     free(msg);
 }
@@ -629,6 +644,20 @@ void InputContext::candidate_activate_cb(void *ptr, int nr, int display_limit)
     ic->candidate_activate(nr, display_limit);
 }
 
+#if UIM_XIM_USE_DELAY
+void InputContext::candidate_activate_with_delay_cb(void *ptr, int delay)
+{
+    InputContext *ic = (InputContext *)ptr;
+    ic->candidate_activate_with_delay(delay);
+}
+
+void InputContext::candidate_activate_timeout_cb(void *ptr)
+{
+    InputContext *ic = (InputContext *)ptr;
+    ic->candidate_activate_timeout();
+}
+#endif
+
 void InputContext::candidate_select_cb(void *ptr, int index)
 {
     InputContext *ic = (InputContext *)ptr;
@@ -808,6 +837,7 @@ void InputContext::candidate_activate(int nr, int display_limit)
 #if !UIM_XIM_USE_NEW_PAGE_HANDLING
     const char *cand_str;
     const char *heading_label;
+    const char *annotation_str;
     char *str;
 #else
     std::vector<CandList>::iterator slot_it;
@@ -815,6 +845,9 @@ void InputContext::candidate_activate(int nr, int display_limit)
     std::vector<const char *> candidates;
     std::vector<const char *>::iterator it;
 
+#if UIM_XIM_USE_DELAY
+    timer_cancel();
+#endif
     Canddisp *disp = canddisp_singleton();
 
     mDisplayLimit = display_limit;
@@ -835,15 +868,15 @@ void InputContext::candidate_activate(int nr, int display_limit)
 			display_limit ? i % display_limit : i);
 	cand_str = uim_candidate_get_cand_str(cand);
 	heading_label = uim_candidate_get_heading_label(cand);
-	//annotation_str = uim_candidate_get_annotation(cand);
-	if (cand_str && heading_label) {
-	    str = (char *)malloc(strlen(cand_str) + strlen(heading_label) + 2);
-	    sprintf(str, "%s\t%s", heading_label, cand_str);
+	annotation_str = uim_candidate_get_annotation(cand);
+	if (cand_str && heading_label && annotation_str) {
+	    str = (char *)malloc(strlen(cand_str) + strlen(heading_label) + strlen(annotation_str) + 3);
+	    sprintf(str, "%s\a%s\a%s", heading_label, cand_str, annotation_str);
 	    candidates.push_back((const char *)str);
 	}
 	else {
 	    fprintf(stderr, "Warning: cand_str at %d is NULL\n", i);
-	    candidates.push_back((const char *)strdup("\t"));
+	    candidates.push_back((const char *)strdup("\a\a"));
 	}
 	uim_candidate_free(cand);
     }
@@ -878,6 +911,30 @@ void InputContext::candidate_activate(int nr, int display_limit)
     need_hilite_selected_cand = false;
 }
 
+#if UIM_XIM_USE_DELAY
+void InputContext::candidate_activate_with_delay(int delay)
+{
+    timer_cancel();
+    if (delay > 0) {
+	timer_set(delay, InputContext::candidate_activate_timeout_cb, this);
+    } else {
+	candidate_activate_timeout();
+    }
+}
+
+void InputContext::candidate_activate_timeout()
+{
+    int nr = -1, display_limit = -1, selected_index = -1;
+    uim_delay_activating(mUc, &nr, &display_limit, &selected_index);
+    if (nr > 0) {
+	candidate_activate(nr, display_limit);
+	if (selected_index >= 0) {
+	    candidate_select(selected_index);
+	}
+    }
+}
+#endif
+
 void InputContext::candidate_update()
 {
     Canddisp *disp = canddisp_singleton();
@@ -901,6 +958,7 @@ void InputContext::prepare_page_candidates(int page)
     int page_nr, start;
     const char *cand_str;
     const char *heading_label;
+    const char *annotation_str;
     char *str;
     CandList candidates;
 
@@ -923,15 +981,15 @@ void InputContext::prepare_page_candidates(int page)
 					(i + start));
 	cand_str = uim_candidate_get_cand_str(cand);
 	heading_label = uim_candidate_get_heading_label(cand);
-	//annotation_str = uim_candidate_get_annotation(cand);
-	if (cand_str && heading_label) {
-	    str = (char *)malloc(strlen(cand_str) + strlen(heading_label) + 2);
-	    sprintf(str, "%s\t%s", heading_label, cand_str);
+	annotation_str = uim_candidate_get_annotation_str(cand);
+	if (cand_str && heading_label && annotation_str) {
+	    str = (char *)malloc(strlen(cand_str) + strlen(heading_label) + strlen(annotation_str) + 3);
+	    sprintf(str, "%s\a%s\a%s", heading_label, cand_str, annotation_str);
 	    candidates.push_back((const char *)str);
 	}
 	else {
 	    fprintf(stderr, "Warning: cand_str at %d is NULL\n", i);
-	    candidates.push_back((const char *)strdup("\t"));
+	    candidates.push_back((const char *)strdup("\a\a"));
 	}
 	uim_candidate_free(cand);
     }
@@ -972,7 +1030,7 @@ void InputContext::candidate_select(int index)
 void InputContext::candidate_shift_page(int direction)
 {
     int new_page;
-    uint new_index;
+    int new_index;
 
     if (mDisplayLimit) {
 	if (direction)
@@ -1011,6 +1069,9 @@ void InputContext::candidate_shift_page(int direction)
 
 void InputContext::candidate_deactivate()
 {
+#if UIM_XIM_USE_DELAY
+    timer_cancel();
+#endif
     if (mCandwinActive) {
 	std::vector<const char *>::iterator i;
 	Canddisp *disp = canddisp_singleton();
@@ -1057,7 +1118,7 @@ char *InputContext::get_caret_state_label_from_prop_list(const char *str)
 	if (p) {
 	    p++;
 	    q = strchr(p, '\t');
-	    len = q - p;
+	    len = static_cast<int>(q - p);
 	    if (q && len < 10) {
 		strlcpy(label, p, len + 1);
 		if (!state_label) {
@@ -1084,9 +1145,10 @@ void InputContext::update_prop_list(const char *str)
 {
     char *buf;
 
-    asprintf(&buf, "prop_list_update\ncharset=UTF-8\n%s", str);
-    if (!buf)
-	return;
+    if (asprintf(&buf, "prop_list_update\ncharset=UTF-8\n%s", str) == -1) {
+        free(buf);
+        return;
+    }
     uim_helper_send_message(lib_uim_fd, buf);
     free(buf);
 
@@ -1096,18 +1158,31 @@ void InputContext::update_prop_list(const char *str)
     // mismatch during IM switch caused from context-update-widgets.
     uim_bool show_caret_state =
 	uim_scm_symbol_value_bool("bridge-show-input-state?");
-    if (show_caret_state == UIM_TRUE) {
+    char *show_caret_with =
+	uim_scm_c_symbol(uim_scm_symbol_value("bridge-show-with?"));
+    uim_bool show_caret_mode = (strcmp(show_caret_with, "mode") == 0);
+    uim_bool show_caret_mode_on = uim_scm_symbol_value_bool("bridge-show-input-state-mode-on?");
+
+    if (show_caret_state == UIM_TRUE && !(show_caret_mode && !show_caret_mode_on)) {
 	char *label;
 	int timeout;
 	Canddisp *disp = canddisp_singleton();
 
-	timeout =
-	    uim_scm_symbol_value_int("bridge-show-input-state-time-length");
+	if (strcmp(show_caret_with, "time") == 0)
+	    timeout = static_cast<int>(uim_scm_symbol_value_int(
+				    "bridge-show-input-state-time-length"));
+	else
+	    timeout = 0;
+
 	label = get_caret_state_label_from_prop_list(str);
 	disp->show_caret_state(label, timeout);
 	free(label);
 	mCaretStateShown = true;
+    } else if (show_caret_mode && !show_caret_mode_on) {
+	Canddisp *disp = canddisp_singleton();
+	disp->hide_caret_state();
     }
+    free(show_caret_with);
 #endif
 }
 
@@ -1115,9 +1190,10 @@ void InputContext::update_prop_label(const char *str)
 {
     char *buf;
 
-    asprintf(&buf, "prop_label_update\ncharset=UTF-8\n%s", str);
-    if (!buf)
-	return;
+    if (asprintf(&buf, "prop_label_update\ncharset=UTF-8\n%s", str) == -1) {
+        free(buf);
+        return;
+    }
     uim_helper_send_message(lib_uim_fd, buf);
     free(buf);
 #if 0    
@@ -1240,62 +1316,8 @@ void keyState::check_key(keyEventX *x)
     if (x->state & Mod5Mask)
 	mModifier |= (gMod5Mask & mPreModState);
 
-    if (x->key_sym < 128 && x->key_sym >= 32)
-	mKey = x->key_sym;
-    else if (x->key_sym >= XK_F1 && x->key_sym <= XK_F35)
-	mKey = x->key_sym - XK_F1 + UKey_F1;
-    // GTK+ and Qt don't support dead_stroke yet
-    else if (x->key_sym >= XK_dead_grave && x->key_sym <= XK_dead_horn)
-	mKey = x->key_sym - XK_dead_grave + UKey_Dead_Grave;
-    else if (x->key_sym >= XK_Kanji && x->key_sym <= XK_Eisu_toggle)
-	mKey = x->key_sym - XK_Kanji + UKey_Kanji;
-    else if (x->key_sym >= XK_Hangul && x->key_sym <= XK_Hangul_Special)
-	mKey = x->key_sym - XK_Hangul + UKey_Hangul;
-    else if (x->key_sym >= XK_kana_fullstop && x->key_sym <= XK_semivoicedsound)
-	mKey = x->key_sym - XK_kana_fullstop + UKey_Kana_Fullstop;
-    else {
-	switch (x->key_sym) {
-	case XK_yen: mKey = UKey_Yen; break;
-	case XK_BackSpace: mKey = UKey_Backspace; break;
-	case XK_Delete: mKey = UKey_Delete; break;
-	case XK_Insert: mKey = UKey_Insert; break;
-	case XK_Escape: mKey = UKey_Escape; break;
-	case XK_Tab:
-	case XK_ISO_Left_Tab: mKey = UKey_Tab; break;
-	case XK_Return: mKey = UKey_Return; break;
-	case XK_Left: mKey = UKey_Left; break;
-	case XK_Up: mKey = UKey_Up; break;
-	case XK_Right: mKey = UKey_Right; break;
-	case XK_Down: mKey = UKey_Down; break;
-	case XK_Prior: mKey = UKey_Prior; break;
-	case XK_Next: mKey = UKey_Next; break;
-	case XK_Home: mKey = UKey_Home; break;
-	case XK_End: mKey = UKey_End; break;
-	case XK_Multi_key: mKey = UKey_Multi_key; break;
-	case XK_Codeinput: mKey = UKey_Codeinput; break;
-	case XK_SingleCandidate: mKey = UKey_SingleCandidate; break;
-	case XK_MultipleCandidate: mKey = UKey_MultipleCandidate; break;
-	case XK_PreviousCandidate: mKey = UKey_PreviousCandidate; break;
-	case XK_Mode_switch: mKey = UKey_Mode_switch; break;
-	case XK_Shift_L: mKey = UKey_Shift_key; break;
-	case XK_Shift_R: mKey = UKey_Shift_key; break;
-	case XK_Control_L: mKey = UKey_Control_key; break;
-	case XK_Control_R: mKey = UKey_Control_key; break;
-	case XK_Alt_L: mKey = UKey_Alt_key; break;
-	case XK_Alt_R: mKey = UKey_Alt_key; break;
-	case XK_Meta_L: mKey = UKey_Meta_key; break;
-	case XK_Meta_R: mKey = UKey_Meta_key; break;
-	case XK_Super_L: mKey = UKey_Super_key; break;
-	case XK_Super_R: mKey = UKey_Super_key; break;
-	case XK_Hyper_L: mKey = UKey_Hyper_key; break;
-	case XK_Hyper_R: mKey = UKey_Hyper_key; break;
-	case XK_Caps_Lock: mKey = UKey_Caps_Lock; break;
-	case XK_Num_Lock: mKey = UKey_Num_Lock; break;
-	case XK_Scroll_Lock: mKey = UKey_Scroll_Lock; break;
-	default:
-	    mKey = UKey_Other;
-	}
-    }
+    mKey = uim_x_keysym2ukey(x->key_sym);
+
 #if UIM_XIM_USE_JAPANESE_KANA_KEYBOARD_HACK
     mKey = uim_x_kana_input_hack_translate_key(mKey,
 					       (KeyCode)x->ev.xkey.keycode);
@@ -1383,8 +1405,9 @@ void init_modifier_keys() {
     gXNumLockMask = 0;
     XModifierKeymap *map = XGetModifierMapping(XimServer::gDpy);
     XDisplayKeycodes(XimServer::gDpy, &min_keycode, &max_keycode);
-    KeySym *sym = XGetKeyboardMapping(XimServer::gDpy, min_keycode,
-		    (max_keycode - min_keycode + 1), &keysyms_per_keycode);
+    KeySym *sym = XGetKeyboardMapping(XimServer::gDpy,
+                    static_cast<KeyCode>(min_keycode),
+                    (max_keycode - min_keycode + 1), &keysyms_per_keycode);
     for (i = 0; i < 8; i++) {
 	int j;
 	for (j = 0; j < map->max_keypermod; j++) {
@@ -1392,8 +1415,8 @@ void init_modifier_keys() {
 		KeySym ks;
 		int index = 0;
 		do {
-		    ks = XKeycodeToKeysym(XimServer::gDpy,
-				    map->modifiermap[k], index);
+		    ks = XkbKeycodeToKeysym(XimServer::gDpy,
+				    map->modifiermap[k], 0, index);
 		    index++;
 		} while (!ks && index < keysyms_per_keycode);
 
@@ -1424,12 +1447,32 @@ void init_modifier_keys() {
     gMod4Mask = check_modifier(Mod4MaskSyms);
     gMod5Mask = check_modifier(Mod5MaskSyms);
 
+    if (uim_scm_c_bool(uim_scm_callf("require-dynlib", "s", "xkb")))
+	uim_scm_callf("%xkb-set-display", "p", XimServer::gDpy);
+
 #if UIM_XIM_USE_JAPANESE_KANA_KEYBOARD_HACK
     // Init at here to sync with proper update timing although not a modifier.
     uim_x_kana_input_hack_init(XimServer::gDpy);
 #endif
 }
 
+void
+check_candwin_style()
+{
+    char *style = uim_scm_symbol_value_str("candidate-window-style");
+    CandWinStyle PrevStyle = XimServer::gCandWinStyle;
+
+    if (style && !strcmp(style, "table"))
+	XimServer::gCandWinStyle = Table;
+    else if (style && !strcmp(style, "horizontal"))
+	XimServer::gCandWinStyle = Horizontal;
+    else
+	XimServer::gCandWinStyle = Vertical;
+
+    XimServer::gCandWinStyleUpdated =
+	    (PrevStyle != XimServer::gCandWinStyle) ? true : false;
+    free(style);
+}
 
 void
 check_candwin_pos_type()
